from .create import CreateView
from .delete import DeleteView
from .detail import DetailView
from .list import ListView
from .update import UpdateView

__all__ = [
    "CreateView",
    "DeleteView",
    "DetailView",
    "ListView",
    "UpdateView",
]
